@@ -8,6 +8,7 @@ import logging
 import itertools
 import functools
 import fractions
+from typing import List, Tuple
 import parsimonious
 from parsimonious.nodes import Node as _Node
 from . import _error
@@ -155,9 +156,9 @@ class _ParseTreeProcessor(parsimonious.NodeVisitor):
         self._comment_is_header = False
         self._comment = ""
 
-    def generic_visit(self, node: _Node, children: typing.Sequence[typing.Any]) -> typing.Any:
+    def generic_visit(self, node: _Node, visited_children: typing.Sequence[typing.Any]) -> typing.Any:
         """If the node has children, replace the node with them."""
-        return tuple(children) or node
+        return tuple(visited_children) or node
 
     def visit_line(self, node: _Node, children: _Children) -> None:
         if len(node.text) == 0:
@@ -301,8 +302,8 @@ class _ParseTreeProcessor(parsimonious.NodeVisitor):
     visit_op2_mul = parsimonious.NodeVisitor.lift_child
     visit_op2_exp = parsimonious.NodeVisitor.lift_child
 
-    def visit_expression_list(self, _n: _Node, children: _Children) -> typing.Tuple[_expression.Any, ...]:
-        out = []  # type: typing.List[_expression.Any]
+    def visit_expression_list(self, _n: _Node, children: _Children) -> Tuple[_expression.Any, ...]:
+        out = []  # type: List[_expression.Any]
         if children:
             children = children[0]
             assert len(children) == 2
